@@ -59,6 +59,7 @@ use core::sync::atomic::{self, Ordering};
 /// - `src` must be valid for reads.
 /// - `src` must be properly aligned.
 /// - `src` must go through [`UnsafeCell::get`](core::cell::UnsafeCell::get).
+/// - `T` must not contain uninitialized bytes.
 /// - There are no concurrent non-atomic write operations.
 /// - There are no concurrent atomic write operations of different
 ///   granularity. The granularity of atomic operations is an implementation
@@ -126,6 +127,7 @@ pub unsafe fn atomic_load<T>(src: *const T, order: Ordering) -> core::mem::Maybe
 /// - `dst` must be [valid] for writes.
 /// - `dst` must be properly aligned.
 /// - `dst` must go through [`UnsafeCell::get`](core::cell::UnsafeCell::get).
+/// - `T` must not contain uninitialized bytes.
 /// - There are no concurrent non-atomic operations.
 /// - There are no concurrent atomic operations of different
 ///   granularity. The granularity of atomic operations is an implementation
@@ -197,9 +199,12 @@ pub unsafe fn atomic_store<T>(dst: *mut T, val: T, order: Ordering) {
 /// Note that the use of `--cfg atomic_memcpy_unsafe_volatile` is
 /// undefined behavior in the multi-threaded environment, since volatile
 /// read/write does not guarantee anything about data race.
-#[cfg(not(any(target_pointer_width = "16", atomic_memcpy_unsafe_volatile)))]
+#[cfg(not(atomic_memcpy_unsafe_volatile))]
 // Note: riscv_target_feature is not stable, so `target_feature = "a"` does not work on stable.
-#[cfg_attr(all(target_arch = "riscv32", target_os = "none"), cfg(target_has_atomic = "ptr"))]
+#[cfg_attr(
+    any(target_pointer_width = "16", all(target_arch = "riscv32", target_os = "none")),
+    cfg(target_has_atomic = "ptr")
+)]
 mod imp {
     #[cfg(not(target_pointer_width = "16"))]
     use core::sync::atomic::AtomicU32;
@@ -389,6 +394,7 @@ mod imp {
         // - `src` is valid for atomic reads.
         // - `src` is properly aligned for `T`.
         // - `src` go through `UnsafeCell::get`.
+        // - `T` does not contain uninitialized bytes.
         // - there are no concurrent non-atomic write operations.
         // - there are no concurrent atomic write operations of different granularity.
         // Note that the safety of the code in this function relies on these guarantees,
@@ -627,6 +633,7 @@ mod imp {
         // - `dst` is valid for atomic writes.
         // - `dst` is properly aligned for `T`.
         // - `dst` go through `UnsafeCell::get`.
+        // - `T` does not contain uninitialized bytes.
         // - there are no concurrent non-atomic operations.
         // - there are no concurrent atomic operations of different granularity.
         // - if there are concurrent atomic write operations, `T` is valid for all bit patterns.
@@ -817,7 +824,10 @@ mod imp {
     all(target_arch = "riscv32", target_os = "none"),
     atomic_memcpy_unsafe_volatile
 ))]
-#[cfg_attr(all(target_arch = "riscv32", target_os = "none"), cfg(not(target_has_atomic = "ptr")))]
+#[cfg_attr(
+    any(target_pointer_width = "16", all(target_arch = "riscv32", target_os = "none")),
+    cfg(not(target_has_atomic = "ptr"))
+)]
 mod imp {
     #[cfg_attr(feature = "inline-always", inline(always))]
     #[cfg_attr(not(feature = "inline-always"), inline)]
